package test;

import java.util.HashSet;
import java.util.List;
import java.util.Set;

import org.eclipse.persistence.dynamic.DynamicClassLoader;
import org.eclipse.persistence.dynamic.DynamicEntity;
import org.eclipse.persistence.dynamic.DynamicHelper;
import org.eclipse.persistence.dynamic.DynamicType;
import org.eclipse.persistence.expressions.Expression;
import org.eclipse.persistence.internal.sessions.DatabaseSessionImpl;
import org.eclipse.persistence.mappings.DatabaseMapping;
import org.eclipse.persistence.queries.ReadAllQuery;
import org.eclipse.persistence.sessions.DatabaseLogin;
import org.eclipse.persistence.sessions.DatabaseSession;

public class EntityContainer
{
    DynamicHelper helper;

    public EntityContainer(DatabaseLogin login)
    {
        DatabaseSession session = new DatabaseSessionImpl(login);
        session.login();
        helper = new DynamicHelper(session);
    }

    public DatabaseSession getDatabaseSession()
    {
        return helper.getSession();
    }

    public DynamicClassLoader getDynamicClassLoader()
    {
        return helper.getDynamicClassLoader();
    }

    public void printEntities(List<DynamicEntity> entities)
    {
        for (DynamicEntity entity : entities)
        {
            printEntity(entity, new HashSet<String>());
            System.out.println("");
        }
    }

    public void printEntity(DynamicEntity entity, Set<String> visited)
    {
        if (entity == null)
            return;
        DynamicType entityType = helper.getType(entity.getClass().getName());
        for (DatabaseMapping dbMapping : entityType.getDescriptor().getMappings())
        {
            if (visited.contains(dbMapping.getAttributeName()))
                continue;

            Object attrValue = entity.get(dbMapping.getAttributeName());
            if (dbMapping.isDirectToFieldMapping())
            {
                System.out.println(dbMapping.getAttributeName() + ": " + attrValue);
            }
            else if (dbMapping.isForeignReferenceMapping())
            {
                visited.add(dbMapping.getAttributeName());
                if (dbMapping.isOneToManyMapping())
                {
                    List<DynamicEntity> dynEntitiesList = (List<DynamicEntity>) attrValue;
                    for (DynamicEntity e : dynEntitiesList)
                         printEntity(e,visited);
                }
                else if (dbMapping.isOneToOneMapping())
                {
                    printEntity((DynamicEntity) attrValue, visited);
                }
            }
            else if (dbMapping.isAggregateMapping())
            {
                if (visited.contains(dbMapping.getAttributeName()))
                    return;
                visited.add(dbMapping.getAttributeName());
                printEntity((DynamicEntity) attrValue, visited);
            }
        }
    }

    public DynamicHelper getHelper()
    {
        return helper;
    }

    public void printEntity(DynamicEntity entity)
    {
        printEntity(entity, new HashSet<String>());
        
    }
    
}
